// WatchDog.cc

/* Author:  Marty Kraimer Date: 26JUN1998 */
 
/*************************************************************************
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory, and the Regents of the University of California, as
* Operator of Los Alamos National Laboratory. MPF is distributed
* subject to a Software License Agreement found in file LICENSE that
* is included with this distribution.
*************************************************************************/

#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>
#include <epicsTimer.h>

#include "WatchDog.h"

static epicsTimerQueueActive *ptimerQueue = 0;

class WatchDogPrivate : public epicsTimerNotify
{
public:
    WatchDogPrivate()
        : callback(0), userPvt(0), timer(ptimerQueue->createTimer()) {}
    virtual ~WatchDogPrivate() {timer.destroy();}
    virtual expireStatus expire(const epicsTime &){
        if(callback) (*callback)(userPvt);
        else printf("WatchDogPrivate::expire called but callback is 0\n");
        return(noRestart);
    }
    watchDogCallback callback;
    void *userPvt;
    epicsTimer &timer;
};

WatchDog::WatchDog()
{
    WatchDogPrivate *pWatchDogPrivate = new WatchDogPrivate;
    wdPvt = (void *)pWatchDogPrivate;
}

WatchDog::~WatchDog()
{
    delete (WatchDogPrivate *)wdPvt;
}

void WatchDog::start(int delay, watchDogCallback callback, void *userPvt)
{
    WatchDogPrivate *pWatchDogPrivate = (WatchDogPrivate *)wdPvt;
    pWatchDogPrivate->callback = callback;
    pWatchDogPrivate->userPvt = userPvt;
    pWatchDogPrivate->timer.start(*pWatchDogPrivate,(double)delay);
}

void WatchDog::start(double delay, watchDogCallback callback, void *userPvt)
{
    WatchDogPrivate *pWatchDogPrivate = (WatchDogPrivate *)wdPvt;
    pWatchDogPrivate->callback = callback;
    pWatchDogPrivate->userPvt = userPvt;
    pWatchDogPrivate->timer.start(*pWatchDogPrivate,delay);
}

void WatchDog::cancel()
{
    WatchDogPrivate *pWatchDogPrivate = (WatchDogPrivate *)wdPvt;
    pWatchDogPrivate->timer.cancel();
    pWatchDogPrivate->callback = 0;
    pWatchDogPrivate->userPvt = 0;
}

// Force allocation of ptimerQueue at load time
class WatchDogQueueCreate {
public:
    WatchDogQueueCreate() {
        ptimerQueue = &epicsTimerQueueActive::allocate(
            true,epicsThreadPriorityMedium);
    }
};

static WatchDogQueueCreate WatchDogQueueCreateObj;
