// mpfTestRegister.cpp
/*************************************************************************
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory, and the Regents of the University of California, as
* Operator of Los Alamos National Laboratory. MPF is distributed
* subject to a Software License Agreement found in file LICENSE that
* is included with this distribution.
*************************************************************************/

#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>

#include <iocsh.h>

#include "Message.h"
#include "test.h"

extern "C" {

//clientChar8Array
static const iocshArg clientChar8ArrayArg0 =
    { "server",iocshArgString};
static const iocshArg clientChar8ArrayArg1 =
    { "slot",iocshArgInt};
static const iocshArg clientChar8ArrayArg2 =
    { "arraySize",iocshArgInt};
static const iocshArg clientChar8ArrayArg3 =
    { "delay",iocshArgDouble};
static const iocshArg * const clientChar8ArrayArgs[4]
    = {&clientChar8ArrayArg0,&clientChar8ArrayArg1,
       &clientChar8ArrayArg2,&clientChar8ArrayArg3};
static const iocshFuncDef clientChar8ArrayFuncDef
    = {"initClientChar8Array",4,clientChar8ArrayArgs};
static void clientChar8ArrayCallFunc(const iocshArgBuf *args)
{
    initClientChar8Array(args[0].sval,args[1].ival,
        args[2].ival,args[3].dval);
}

//clientInt32
static const iocshArg clientInt32Arg0 =
    { "server",iocshArgString};
static const iocshArg clientInt32Arg1 =
    { "slot",iocshArgInt};
static const iocshArg clientInt32Arg2 =
    { "ntoBuffer",iocshArgInt};
static const iocshArg clientInt32Arg3 =
    { "delay",iocshArgDouble};
static const iocshArg * const clientInt32Args[4]
    = {&clientInt32Arg0,&clientInt32Arg1,
       &clientInt32Arg2,&clientInt32Arg3};
static const iocshFuncDef clientInt32FuncDef
    = {"initClientInt32",4,clientInt32Args};
static void clientInt32CallFunc(const iocshArgBuf *args)
{
    initClientInt32(args[0].sval,args[1].ival,
        args[2].ival,args[3].dval);
}

//serverChar8Array
static const iocshArg serverChar8ArrayArg0 =
    { "server",iocshArgString};
static const iocshArg serverChar8ArrayArg1 =
    { "queueSize",iocshArgInt};
static const iocshArg * const serverChar8ArrayArgs[2]
    = {&serverChar8ArrayArg0,&serverChar8ArrayArg1};
static const iocshFuncDef serverChar8ArrayFuncDef
    = {"initServerChar8Array",2,serverChar8ArrayArgs};
static void serverChar8ArrayCallFunc(const iocshArgBuf *args)
{
    initServerChar8Array(args[0].sval,args[1].ival);
}

//serverInt32
static const iocshArg serverInt32Arg0 =
    { "server",iocshArgString};
static const iocshArg serverInt32Arg1 =
    { "queueSize",iocshArgInt};
static const iocshArg * const serverInt32Args[2]
    = {&serverInt32Arg0,&serverInt32Arg1};
static const iocshFuncDef serverInt32FuncDef
    = {"initServerInt32",2,serverInt32Args};
static void serverInt32CallFunc(const iocshArgBuf *args)
{
    initServerInt32(args[0].sval,args[1].ival);
}

//messageTest
static const iocshArg messageTestArg0 =
    { "ntimes",iocshArgInt};
static const iocshArg * const messageTestArgs[1]
    = {&messageTestArg0};
static const iocshFuncDef messageTestFuncDef
    = {"initMessageTest",1,messageTestArgs};
static void messageTestCallFunc(const iocshArgBuf *args)
{
    initMessageTest(args[0].ival);
}

//tcpTest
static const iocshArg tcpTestArg0 =
    { "isServer",iocshArgInt};
static const iocshArg tcpTestArg1 =
    { "port",iocshArgInt};
static const iocshArg tcpTestArg2 =
    { "address",iocshArgString};
static const iocshArg tcpTestArg3 =
    { "bufferSize",iocshArgInt};
static const iocshArg * const tcpTestArgs[4]
    = {&tcpTestArg0,&tcpTestArg1,&tcpTestArg2,&tcpTestArg3};
static const iocshFuncDef tcpTestFuncDef
    = {"initTcpTest",4,tcpTestArgs};
static void tcpTestCallFunc(const iocshArgBuf *args)
{
    initTcpTest(args[0].ival,args[1].ival,
        args[2].sval,args[3].ival);
}

//tcpTestWait
static const iocshArg tcpTestWaitArg0 =
    { "isServer",iocshArgInt};
static const iocshArg tcpTestWaitArg1 =
    { "port",iocshArgInt};
static const iocshArg tcpTestWaitArg2 =
    { "address",iocshArgString};
static const iocshArg tcpTestWaitArg3 =
    { "bufferSize",iocshArgInt};
static const iocshArg tcpTestWaitArg4 =
    { "nvalues",iocshArgInt};
static const iocshArg * const tcpTestWaitArgs[5]
    = {&tcpTestWaitArg0,&tcpTestWaitArg1,
       &tcpTestWaitArg2,&tcpTestWaitArg3,&tcpTestWaitArg4};
static const iocshFuncDef tcpTestWaitFuncDef
    = {"initTcpTestWait",5,tcpTestWaitArgs};
static void tcpTestWaitCallFunc(const iocshArgBuf *args)
{
    initTcpTestWait(args[0].ival,args[1].ival,
        args[2].sval,args[3].ival,args[4].ival);
}

//clientChar8ArrayDebug
static const iocshArg clientChar8ArrayDebugArg0 = { "value",iocshArgInt};
static const iocshArg * const clientChar8ArrayDebugArgs[1] = {&clientChar8ArrayDebugArg0};
static const iocshFuncDef clientChar8ArrayDebugFuncDef = {"setClientChar8ArrayDebug",1,clientChar8ArrayDebugArgs};
static void clientChar8ArrayDebugCallFunc(const iocshArgBuf *args)
{
    clientChar8ArrayDebug = args[0].ival;
}

//clientInt32Debug
static const iocshArg clientInt32DebugArg0 = { "value",iocshArgInt};
static const iocshArg * const clientInt32DebugArgs[1] = {&clientInt32DebugArg0};
static const iocshFuncDef clientInt32DebugFuncDef = {"setClientInt32Debug",1,clientInt32DebugArgs};
static void clientInt32DebugCallFunc(const iocshArgBuf *args)
{
    clientInt32Debug = args[0].ival;
}

//serverChar8ArrayDebug
static const iocshArg serverChar8ArrayDebugArg0 = { "value",iocshArgInt};
static const iocshArg * const serverChar8ArrayDebugArgs[1] = {&serverChar8ArrayDebugArg0};
static const iocshFuncDef serverChar8ArrayDebugFuncDef = {"setServerChar8ArrayDebug",1,serverChar8ArrayDebugArgs};
static void serverChar8ArrayDebugCallFunc(const iocshArgBuf *args)
{
    serverChar8ArrayDebug = args[0].ival;
}

//serverInt32Debug
static const iocshArg serverInt32DebugArg0 = { "value",iocshArgInt};
static const iocshArg * const serverInt32DebugArgs[1] = {&serverInt32DebugArg0};
static const iocshFuncDef serverInt32DebugFuncDef = {"setServerInt32Debug",1,serverInt32DebugArgs};
static void serverInt32DebugCallFunc(const iocshArgBuf *args)
{
    serverInt32Debug = args[0].ival;
}

//tcpTestDebug
static const iocshArg tcpTestDebugArg0 = { "value",iocshArgInt};
static const iocshArg * const tcpTestDebugArgs[1] = {&tcpTestDebugArg0};
static const iocshFuncDef tcpTestDebugFuncDef = {"setTcpTestDebug",1,tcpTestDebugArgs};
static void tcpTestDebugCallFunc(const iocshArgBuf *args)
{
    tcpTestDebug = args[0].ival;
}

// tcpTestStats
static const iocshFuncDef tcpTestStatsFuncDef = {"tcpTestStats",0,0};
static void tcpTestStatsCallFunc(const iocshArgBuf *)
{
    tcpTestStats();
}

//tcpTestWaitDebug
static const iocshArg tcpTestWaitDebugArg0 = { "value",iocshArgInt};
static const iocshArg * const tcpTestWaitDebugArgs[1] = {&tcpTestWaitDebugArg0};
static const iocshFuncDef tcpTestWaitDebugFuncDef = {"setTcpTestWaitDebug",1,tcpTestWaitDebugArgs};
static void tcpTestWaitDebugCallFunc(const iocshArgBuf *args)
{
    tcpTestWaitDebug = args[0].ival;
}

//tcpTestWaitNvalues
static const iocshArg tcpTestWaitNvaluesArg0 = { "value",iocshArgInt};
static const iocshArg * const tcpTestWaitNvaluesArgs[1] = {&tcpTestWaitNvaluesArg0};
static const iocshFuncDef tcpTestWaitNvaluesFuncDef = {"setTcpTestWaitNvalues",1,tcpTestWaitNvaluesArgs};
static void tcpTestWaitNvaluesCallFunc(const iocshArgBuf *args)
{
    tcpTestWaitNvalues = args[0].ival;
}

// tcpTestWaitStats
static const iocshFuncDef tcpTestWaitStatsFuncDef = {"tcpTestWaitStats",0,0};
static void tcpTestWaitStatsCallFunc(const iocshArgBuf *)
{
    tcpTestWaitStats();
}


void mpfTestRegister(void)
{
    iocshRegister(&clientChar8ArrayFuncDef,clientChar8ArrayCallFunc);
    iocshRegister(&clientInt32FuncDef,clientInt32CallFunc);
    iocshRegister(&serverChar8ArrayFuncDef,serverChar8ArrayCallFunc);
    iocshRegister(&serverInt32FuncDef,serverInt32CallFunc);
    iocshRegister(&messageTestFuncDef,messageTestCallFunc);
    iocshRegister(&tcpTestFuncDef,tcpTestCallFunc);
    iocshRegister(&tcpTestWaitFuncDef,tcpTestWaitCallFunc);
    iocshRegister(&clientChar8ArrayDebugFuncDef,clientChar8ArrayDebugCallFunc);
    iocshRegister(&clientInt32DebugFuncDef,clientInt32DebugCallFunc);
    iocshRegister(&serverChar8ArrayDebugFuncDef,serverChar8ArrayDebugCallFunc);
    iocshRegister(&serverInt32DebugFuncDef,serverInt32DebugCallFunc);
    iocshRegister(&tcpTestDebugFuncDef,tcpTestDebugCallFunc);
    iocshRegister(&tcpTestStatsFuncDef,tcpTestStatsCallFunc);
    iocshRegister(&tcpTestWaitDebugFuncDef,tcpTestWaitDebugCallFunc);
    iocshRegister(&tcpTestWaitNvaluesFuncDef,tcpTestWaitNvaluesCallFunc);
    iocshRegister(&tcpTestWaitStatsFuncDef,tcpTestWaitStatsCallFunc);
}

} // extern "C"

class MpfTestRegister {
public:
    MpfTestRegister() {mpfTestRegister();}
};

static MpfTestRegister mpfTestRegisterObj;
