// tcpTest.cpp
/*************************************************************************
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory, and the Regents of the University of California, as
* Operator of Los Alamos National Laboratory. MPF is distributed
* subject to a Software License Agreement found in file LICENSE that
* is included with this distribution.
*************************************************************************/

#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>
#include <epicsThread.h>

#include "mpfType.h"
#include "Tcp.h"
#include "test.h"

typedef struct privateData {
    tcpConnectState connectState;
    long	value;
    bool isServer;
}privateData;

int tcpTestDebug = 0;

Tcp *plasttcpTest = 0;
 
int tcpTestStats()
{
    if(!plasttcpTest) {
        printf("No Tcp thread\n");
    } else {
        plasttcpTest->printStats();
    }
    return(0);
}

static const char *isServerString[2] = {"no","yes"};
static void connectCallback(void *pvt,tcpConnectState state)
{
    privateData *pprivateData = (privateData *)pvt;
    
    if(tcpTestDebug)printf("tcpTest connectCallback server %s state %d\n",
	isServerString[(pprivateData->isServer)?1:0],(int)state);
    pprivateData->connectState = state;
}

static void receiveCallback(void *pvt,void *buffer,int32 nbytes)
{
    privateData *pprivateData = (privateData *)pvt;
    
    if(tcpTestDebug)printf("tcpTest receiveCallback server %s ",
	isServerString[(pprivateData->isServer)?1:0]);
    int32 *pvalue = (int32 *)buffer;
    if(tcpTestDebug)printf("value %d nbytes %d\n",*pvalue,nbytes);
}
typedef struct args {
    int server;
    int port;
    const char *address;
    int bufferSize;
}args;

static void tcpTest(void *inargs)
{
    args *pargs = (args *)inargs;
    int server = pargs->server;
    int port = pargs->port;
    const char *address = pargs->address;
    int bufferSize = pargs->bufferSize;
    bool isServer = (server? true : false);

    privateData	*pprivateData = new privateData;
    pprivateData->isServer = isServer;
    Tcp *pTcp = new Tcp(isServer,port,address,bufferSize);
    plasttcpTest = pTcp;
    pTcp->connect(pprivateData,connectCallback,receiveCallback);
    int32 value = 0;
    while(true) {
        if(pprivateData->connectState!=tcpConnectYes) {
	    epicsThreadSleep(10.0);
	    continue;
	}
	pTcp->send((void *)&value,sizeof(value));
	value++;
    }
}

extern "C" int initTcpTest(int server,int port,const char *address,int bufferSize)
{
    args *pargs = (args *)calloc(1,sizeof(args));
    epicsThreadId threadId;

    pargs->server = server;
    pargs->port = port;
    pargs->address = address;
    pargs->bufferSize = bufferSize;
    threadId = epicsThreadCreate("tcpTest",epicsThreadPriorityMedium,
        epicsThreadGetStackSize(epicsThreadStackMedium),
        (EPICSTHREADFUNC)tcpTest,(void *)pargs);
    if(!threadId) printf("initTcpTest epicsThreadCreate Failure\n");
    return(0);
}
