// tcpTestWait.cpp
/*************************************************************************
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory, and the Regents of the University of California, as
* Operator of Los Alamos National Laboratory. MPF is distributed
* subject to a Software License Agreement found in file LICENSE that
* is included with this distribution.
*************************************************************************/

#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>
#include <epicsEvent.h>

#include <epicsThread.h>

#include "mpfType.h"
#include "Tcp.h"
#include "test.h"

typedef struct privateData {
    tcpConnectState connectState;
    int32	valueSent;
    int32	valueReceived;
    epicsEventId sem;
    Tcp 	*pTcp;
    bool isServer;
    bool justConnected;
}privateData;

int tcpTestWaitDebug = 0;
int tcpTestWaitNvalues = 1;
int tcpTestNreceiveServer = 0;
int tcpTestNreceiveClient = 0;

Tcp *tcpTesttcpTestWait = 0;

int tcpTestWaitStats()
{
    if(!tcpTesttcpTestWait) {
	printf("No Tcp thread\n");
    } else {
	tcpTesttcpTestWait->printStats();
    }
    return(0);
}

static const char *isServerString[2] = {"no","yes"};
static void connectCallback(void *pvt,tcpConnectState state)
{
    privateData *pprivateData = (privateData *)pvt;
    printf("tcpTestWait connectCallback server %s state %d\n",
	    isServerString[(pprivateData->isServer)?1:0],(int)state);
    pprivateData->connectState = state;
    if(state==tcpConnectYes) pprivateData->justConnected = true;
    epicsEventSignal(pprivateData->sem);
}

static void receiveCallbackServer(void *pvt,void *buffer,int32 nbytes)
{
    privateData *pprivateData = (privateData *)pvt;

    
    if(tcpTestWaitDebug)printf("tcpTestWait receiveCallbackServer\n");
    int32 *pvalue = (int32 *)buffer;
    pprivateData->valueReceived = *pvalue;
    if(tcpTestWaitDebug)printf("value %d nbytes %d\n",*pvalue,nbytes);
    tcpTestNreceiveServer++;
    if(tcpTestNreceiveServer==tcpTestWaitNvalues) {
	epicsEventSignal(pprivateData->sem);
	tcpTestNreceiveServer=0;
    }
}

static void server(privateData *pprivateData)
{
    Tcp *pTcp = pprivateData->pTcp;
    pTcp->connect(pprivateData,connectCallback,receiveCallbackServer);
    while(true) {
	epicsEventWaitWithTimeout(pprivateData->sem,10.0);
	if(pprivateData->connectState==tcpConnectYes) {
	    if(pprivateData->justConnected) {
		pprivateData->justConnected = false;
		continue;
	    }
	    pprivateData->valueSent = pprivateData->valueReceived;
	    int status = 0;
	    for(int i=0; i<tcpTestWaitNvalues; i++) {
		status = pTcp->send((void *)&pprivateData->valueSent,
		    sizeof(pprivateData->valueSent));
		if(status) break;
	    }
	    if(!status) status = pTcp->flush();
	    if(status) {
		printf("sendFlush failed in server\n");
	    }
	} else {
	    printf("server failed\n");
	}
    }
}

static void receiveCallbackClient(void *pvt,void *buffer,int32 nbytes)
{
    privateData *pprivateData = (privateData *)pvt;
    
    if(tcpTestWaitDebug)printf("tcpTestWait receiveCallbackClient\n");
    int32 *pvalue = (int32 *)buffer;
    pprivateData->valueReceived = *pvalue;
    if(pprivateData->valueReceived != pprivateData->valueSent){
       printf("receiveCallback sent %d received %d\n",
	   pprivateData->valueSent,pprivateData->valueReceived);
    } else {
        if(tcpTestWaitDebug)printf("value %d nbytes %d\n",*pvalue,nbytes);
    }
    tcpTestNreceiveClient++;
    if(tcpTestNreceiveClient==tcpTestWaitNvalues) {
	epicsEventSignal(pprivateData->sem);
	tcpTestNreceiveClient=0;
    }
}

static void client(privateData *pprivateData)
{
    Tcp *pTcp = pprivateData->pTcp;
    pTcp->connect(pprivateData,connectCallback,receiveCallbackClient);
    //wait for connect
    epicsEventWaitWithTimeout(pprivateData->sem,10.0);
    pprivateData->valueReceived = 0;
    pprivateData->valueSent = 1;
    while(true) {
	epicsEventWaitStatus semStatus;
        semStatus = epicsEventWaitWithTimeout(pprivateData->sem,10.0);
	if(pprivateData->connectState==tcpConnectNo) continue;
	if(semStatus==epicsEventWaitOK) {
	    int nwait=0;
	    while(pprivateData->valueSent!=pprivateData->valueReceived) {
		if(nwait==0)printf("client send %d receive %d\n",
		    pprivateData->valueSent,pprivateData->valueReceived);
		//wait an extra second for more messages
		epicsEventWaitWithTimeout(pprivateData->sem,1.0);
		if(nwait++ == 5) break;
	    }
	    if(pprivateData->valueSent==pprivateData->valueReceived)
		++pprivateData->valueSent;
	}
	int status = 0;
	for(int i=0; i<tcpTestWaitNvalues; i++) {
	    status = pTcp->send((void *)&pprivateData->valueSent,
		sizeof(pprivateData->valueSent));
	}
	if(!status)status = pTcp->flush();
	if(status) {
	    printf("sendFlush failed in client\n");
	}
    }
}

typedef struct args {
    int isserver;
    int port;
    const char *address;
    int bufferSize;
    int nvalues;
}args;

static void tcpTestWait(void *inargs)
{
    args *pargs = (args *)inargs;
    int isserver = pargs->isserver;
    int port = pargs->port;
    const char *address = pargs->address;
    int bufferSize = pargs->bufferSize;
    int nvalues = pargs->nvalues;

    if(nvalues>0) tcpTestWaitNvalues = nvalues;
    bool isServer = (isserver? true : false);
    privateData	*pprivateData = new privateData;
    pprivateData->sem  = epicsEventMustCreate(epicsEventEmpty);
    pprivateData->isServer = isServer;
    Tcp *pTcp = new Tcp(isServer,port,address,bufferSize);
    tcpTesttcpTestWait = pTcp;
    pprivateData->pTcp = pTcp;
    if(isServer)
	server(pprivateData);
    else
	client(pprivateData);
    return;
}

extern "C" int initTcpTestWait(int isserver,int port,const char *address,
    int bufferSize,int nvalues)
{
    args *pargs = (args *)calloc(1,sizeof(args));
    epicsThreadId threadId;

    pargs->isserver = isserver;
    pargs->port = port;
    pargs->address = address;
    pargs->bufferSize = bufferSize;
    pargs->nvalues = nvalues;
    threadId = epicsThreadCreate("tcpTestWait",epicsThreadPriorityMedium,
        epicsThreadGetStackSize(epicsThreadStackMedium),
        (EPICSTHREADFUNC)tcpTestWait,(void *)pargs);
    if(!threadId) printf("initTcpTest epicsThreadCreate Failure\n");
    return(0);
}
